EXEC [__AddStoredProcInfo]
    /* StoredProc_Name         */ 'RubyGems_CreateOrUpdateGem',
    /* Internal_Indicator      */ 'Y',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Creates a new Ruby Gem or updates an existing one.'
GO

IF OBJECT_ID('[RubyGems_CreateOrUpdateGem]') IS NOT NULL DROP PROCEDURE [RubyGems_CreateOrUpdateGem]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [RubyGems_CreateOrUpdateGem]
(
	@Feed_Id INT,
	@Gem_Name VARCHAR(200),
	@Version_Text VARCHAR(50),
	@Platform_Text VARCHAR(50),
	@Metadata_Bytes VARBINARY(MAX) = NULL,
	@Gem_Size BIGINT = NULL,
	@Summary_Text VARCHAR(200) = NULL,
	@Published_Date DATETIME = NULL,
	@Cached_Indicator YNINDICATOR = NULL,
	@LatestVersion_Text VARCHAR(50) = NULL,
	@GemHash_Bytes BINARY(32) = NULL,
	@Active_Indicator YNINDICATOR = NULL
)
AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

		DECLARE @Gem_Id INT
		SET @Gem_Id = (SELECT [Gem_Id] FROM [RubyGems] WHERE [Feed_Id] = @Feed_Id AND [Gem_Name] = @Gem_Name)

		IF @Gem_Id IS NULL
		BEGIN

			INSERT INTO [RubyGems]
			(
				[Feed_Id],
				[Gem_Name],
				[LatestVersion_Text],
				[Summary_Text]
			)
			VALUES
			(
				@Feed_Id,
				@Gem_Name,
				@LatestVersion_Text,
				@Summary_Text
			)

			SET @Gem_Id = SCOPE_IDENTITY()

		END
		ELSE IF @Version_Text = @LatestVersion_Text
		BEGIN

			UPDATE [RubyGems]
			   SET [Summary_Text] = COALESCE(@Summary_Text, [Summary_Text]),
				   [LatestVersion_Text] = COALESCE(@LatestVersion_Text, [LatestVersion_Text])
			 WHERE [Gem_Id] = @Gem_Id

		END
		ELSE
		BEGIN

			UPDATE [RubyGems]
			   SET [LatestVersion_Text] = COALESCE(@LatestVersion_Text, [LatestVersion_Text])
			 WHERE [Gem_Id] = @Gem_Id

		END

		IF EXISTS(SELECT * FROM [RubyGemVersions] WITH (UPDLOCK) WHERE [Gem_Id] = @Gem_Id AND [Version_Text] = @Version_Text AND [Platform_Text] = @Platform_Text)
		BEGIN

			UPDATE [RubyGemVersions]
			   SET [Metadata_Bytes] = COALESCE(@Metadata_Bytes, [Metadata_Bytes]),
			       [Published_Date] = COALESCE(@Published_Date, [Published_Date]),
				   [Gem_Size] = COALESCE(@Gem_Size, [Gem_Size]),
				   [Cached_Indicator] = COALESCE(@Cached_Indicator, [Cached_Indicator]),
				   [GemHash_Bytes] = COALESCE(@GemHash_Bytes, [GemHash_Bytes]),
				   [Active_Indicator] = COALESCE(@Active_Indicator, [Active_Indicator])
			 WHERE [Gem_Id] = @Gem_Id
			   AND [Version_Text] = @Version_Text
			   AND [Platform_Text] = @Platform_Text

			IF COALESCE(dbo.Session_GetUserName(), 'UNKNOWN') NOT IN ('SYSTEM', 'UNKNOWN')
	   			EXEC [Events_RaiseEvent] 'PKGMDF', @Feed_Id, 'Package_Id', @Gem_Name, 'Package_Version', @Version_Text

		END
		ELSE
		BEGIN

			INSERT INTO [RubyGemVersions]
			(
				[Gem_Id],
				[Version_Text],
				[Platform_Text],
				[Download_Count],
				[Metadata_Bytes],
				[Gem_Size],
				[Published_Date],
				[Cached_Indicator],
				[GemHash_Bytes],
				[Active_Indicator]
			)
			VALUES
			(
				@Gem_Id,
				@Version_Text,
				@Platform_Text,
				0,
				@Metadata_Bytes,
				@Gem_Size,
				COALESCE(@Published_Date, GETUTCDATE()),
				COALESCE(@Cached_Indicator, 'N'),
				@GemHash_Bytes,
				COALESCE(@Active_Indicator, 'Y')
			)

			EXEC [Events_RaiseEvent] 'PKGADD', @Feed_Id, 'Package_Id', @Gem_Name, 'Package_Version', @Version_Text

		END

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [RubyGems_CreateOrUpdateGem] TO [ProGetUser_Role]
GO