EXEC [__AddStoredProcInfo]
    /* StoredProc_Name         */ 'Vulnerabilities_CreateOrUpdateComment',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ 'VulnerabilityComment_Id',
    /* Description_Text        */ 'Creates a new vulnerability comment or edits an existing one.'
GO

IF OBJECT_ID('[Vulnerabilities_CreateOrUpdateComment]') IS NOT NULL DROP PROCEDURE [Vulnerabilities_CreateOrUpdateComment]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Vulnerabilities_CreateOrUpdateComment]
(
	@Vulnerability_Id INT,
	@Comment_Text NVARCHAR(MAX),
	@CommentedBy_User_Name USERNAME,
	@CommentedOn_Date DATETIME,
	@VulnerabilityComment_Id INT = NULL OUT
)
AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

	IF @VulnerabilityComment_Id IS NULL
	BEGIN

		INSERT INTO [VulnerabilityComments]
		(
			[Vulnerability_Id],
			[Comment_Text],
			[CommentedBy_User_Name],
			[CommentedOn_Date]
		)
		VALUES
		(
			@Vulnerability_Id,
			COALESCE(@Comment_Text, ''),
			@CommentedBy_User_Name,
			@CommentedOn_Date
		)

		SET @VulnerabilityComment_Id = SCOPE_IDENTITY()

	END
	ELSE
	BEGIN

		UPDATE [VulnerabilityComments]
		   SET [Comment_Text] = @Comment_Text
		 WHERE [VulnerabilityComment_Id] = @VulnerabilityComment_Id

	END

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [Vulnerabilities_CreateOrUpdateComment] TO [ProGetUser_Role]
GO